"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const server_1 = require("./server");
const commander = require("commander");
const chalk_1 = require("chalk");
const in_memory_client_1 = require("./repositories/in-memory-client");
const in_memory_subscription_1 = require("./repositories/in-memory-subscription");
const mongo_client_1 = require("./repositories/mongo-client");
const mongo_subscription_1 = require("./repositories/mongo-subscription");
const child_process_1 = require("child_process");
const fs = require("fs");
const install_1 = require("./validators/cli/install");
const start_1 = require("./validators/cli/start");
commander
    .command('install')
    .option('-h --host <host>', `defaults to 'your-domain.com'`)
    .option('-m --mongo <host>', 'if not provided, file-based storage will be used')
    .option('-p --port <port>', 'port')
    .option('--nginx', 'install NGINX')
    .option('--letsencrypt', `install and configure Let's Encrypt`)
    .action((command) => {
    if (!install_1.InstallCLIValidator.validate(command)) {
        for (const error of install_1.InstallCLIValidator.errors(command)) {
            console.log(`${chalk_1.default.red('ERROR:')} ${chalk_1.default.white(error)}`);
        }
        return;
    }
    console.log(chalk_1.default.blue(`Creating systemd service file...`));
    fs.writeFileSync('/lib/systemd/system/web-push-service.service', `[Unit]
    Description=Web Push Service
    
    [Service]
    Type=simple
    ExecStart=/usr/bin/web-push-service start --host ${command.host ? command.host : 'your-domain.com'}${command.mongo ? `--mongo ${command.mongo}` : ''} --port ${command.port}
    
    [Install]
    WantedBy=multi-user.target`);
    console.log(chalk_1.default.blue(`Enabling web-push-service`));
    child_process_1.spawnSync('systemctl', ['enable', 'web-push-service']);
    console.log(chalk_1.default.blue(`Starting web-push-service...`));
    child_process_1.spawnSync('systemctl', ['start', 'web-push-service']);
    if (command.letsencrypt) {
        console.log(chalk_1.default.blue(`Installing letsencrypt...`));
        child_process_1.spawnSync('apt-get', ['install', '-y', 'letsencrypt']);
        console.log(chalk_1.default.blue(`Obtaining Certificate letsencrypt...`));
        child_process_1.spawnSync('letsencrypt', [
            'certonly',
            '--standalone',
            '--agree-tos',
            '--email',
            'developersworkspace@gmail.com',
            '-d',
            command.host,
        ]);
    }
    if (command.nginx) {
        console.log(chalk_1.default.blue(`Installing nginx...`));
        child_process_1.spawnSync('apt', ['install', '-y', 'nginx']);
        console.log(chalk_1.default.blue(`Creating nginx configuration file...`));
        fs.writeFileSync('/etc/nginx/sites-enabled/web-push-service', `upstream web-push-service {
      server 127.0.0.1:8080;
  }
  
  server {
      listen 80;
      server_name ${command.host ? command.host : 'your-domain.com'};
  
      server_tokens off;
      gzip on;
      gzip_min_length 1000;
      gunzip on;
      gzip_static on;
  
      location / {
        proxy_pass http://web-push-service;
      }
  }
  
  server {
      listen 443 ssl;
      server_name ${command.host ? command.host : 'your-domain.com'};
  
      server_tokens off;
      gzip on;
      gzip_min_length 1000;
      gunzip on;
      gzip_static on;
  
      proxy_set_header X-Real-IP $remote_addr;

      ${command.letsencrypt
            ? `ssl_certificate /etc/letsencrypt/live/${command.host ? command.host : 'your-domain.com'}/fullchain.pem;
      ssl_certificate_key /etc/letsencrypt/live/${command.host ? command.host : 'your-domain.com'}/privkey.pem;
  
      ssl on;
      ssl_session_cache  builtin:1000  shared:SSL:10m;
      ssl_protocols  TLSv1 TLSv1.1 TLSv1.2;
      ssl_ciphers ECDHE-RSA-AES256-GCM-SHA384:ECDHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-RSA-AES128-SHA256:ECDHE-RSA-AES256-SHA:ECDHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA:DHE-RSA-AES128-SHA;
      ssl_prefer_server_ciphers on;`
            : ''}
      
  
      location / {
          proxy_pass http://web-push-service;
      }
  }`);
        console.log(chalk_1.default.blue(`Configuring firewall...`));
        child_process_1.spawnSync('ufw', ['allow', `'Nginx Full'`]);
        console.log(chalk_1.default.blue(`Enabling nginx...`));
        child_process_1.spawnSync('systemctl', ['enable', 'nginx']);
        console.log(chalk_1.default.blue(`Starting nginx...`));
        child_process_1.spawnSync('systemctl', ['start', 'nginx']);
    }
});
commander
    .command('start')
    .option('-h --host <host>', `defaults to 'localhost:8080'`)
    .option('-m --mongo <host>', 'if not provided, file-based storage will be used')
    .option('-p --port <port>', 'port')
    .action((command) => {
    if (!start_1.StartCLIValidator.validate(command)) {
        for (const error of start_1.StartCLIValidator.errors(command)) {
            console.log(`${chalk_1.default.red('ERROR:')} ${chalk_1.default.white(error)}`);
        }
        return;
    }
    let clientRepository = null;
    let subscriptionRepository = null;
    if (command.mongo) {
        clientRepository = new mongo_client_1.MongoClientRepository(command.mongo);
        subscriptionRepository = new mongo_subscription_1.MongoSubscriptionRepository(command.mongo);
    }
    else {
        clientRepository = new in_memory_client_1.InMemoryClientRepository('./clients.dat');
        subscriptionRepository = new in_memory_subscription_1.InMemorySubscriptionRepository('./subscriptions.dat');
    }
    const expressApplication = server_1.initialize(clientRepository, subscriptionRepository, command.host ? command.host : 'localhost:8080');
    expressApplication.listen(command.port);
    console.log(chalk_1.default.green(`Listening on port ${command.port}`));
});
commander.parse(process.argv);
//# sourceMappingURL=main.js.map