"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const push_1 = require("../../src/services/push");
const sinon = require("sinon");
const webpush = require("web-push");
const subscription_1 = require("../../src/models/subscription");
const client_1 = require("../../src/models/client");
describe('PushService', () => {
    describe('#create', () => {
        it('Should call webpush#sendNotification', () => __awaiter(this, void 0, void 0, function* () {
            const subscriptionRepository = {
                findAll: sinon
                    .stub()
                    .returns(Promise.resolve([new subscription_1.Subscription('endpoint', 0, { auth: 'auth', p256dh: 'p256dh' })])),
            };
            const pushService = new push_1.PushService(subscriptionRepository);
            const webpushSendNotificationSpy = sinon.spy();
            webpush.sendNotification = webpushSendNotificationSpy;
            yield pushService.create(new client_1.Client(null, 'key', 'public-key', 'private-key', 'endpoint'), 'channel', 'payload');
            chai_1.expect(webpushSendNotificationSpy.calledOnce).to.be.true;
        }));
        it('Should call webpush#sendNotification with correct parameters', () => __awaiter(this, void 0, void 0, function* () {
            const subscriptionRepository = {
                findAll: sinon
                    .stub()
                    .returns(Promise.resolve([new subscription_1.Subscription('endpoint', 0, { auth: 'auth', p256dh: 'p256dh' })])),
            };
            const pushService = new push_1.PushService(subscriptionRepository);
            const webpushSendNotificationSpy = sinon.spy();
            webpush.sendNotification = webpushSendNotificationSpy;
            yield pushService.create(new client_1.Client(null, 'key', 'public-key', 'private-key', 'endpoint'), 'channel', 'payload');
            const subscription = webpushSendNotificationSpy.args[0][0];
            const payload = webpushSendNotificationSpy.args[0][1];
            const options = webpushSendNotificationSpy.args[0][2];
            chai_1.expect(subscription.endpoint).to.be.eq('endpoint');
            chai_1.expect(subscription.expirationTime).to.be.eq(0);
            chai_1.expect(subscription.keys.auth).to.be.eq('auth');
            chai_1.expect(subscription.keys.p256dh).to.be.eq('p256dh');
            chai_1.expect(payload).to.be.eq('"payload"');
            chai_1.expect(options.vapidDetails.subject).to.be.eq('endpoint');
            chai_1.expect(options.vapidDetails.publicKey).to.be.eq('public-key');
            chai_1.expect(options.vapidDetails.privateKey).to.be.eq('private-key');
            chai_1.expect(options.TTL).to.be.eq(604800);
        }));
        it('Should throw error given null client', () => __awaiter(this, void 0, void 0, function* () {
            const pushService = new push_1.PushService(null);
            try {
                yield pushService.create(null, null, null);
                throw new Error('Expected Error');
            }
            catch (error) {
                chai_1.expect(error.message).to.be.eq('Client cannot be null');
            }
        }));
        it('Should throw error given client with null key', () => __awaiter(this, void 0, void 0, function* () {
            const pushService = new push_1.PushService(null);
            try {
                yield pushService.create(new client_1.Client(null, null, null, null, null), null, null);
                throw new Error('Expected Error');
            }
            catch (error) {
                chai_1.expect(error.message).to.be.eq('Key cannot be null');
            }
        }));
        it('Should throw error given client with null public key', () => __awaiter(this, void 0, void 0, function* () {
            const pushService = new push_1.PushService(null);
            try {
                yield pushService.create(new client_1.Client(null, 'key', null, null, null), null, null);
                throw new Error('Expected Error');
            }
            catch (error) {
                chai_1.expect(error.message).to.be.eq('Public Key cannot be null');
            }
        }));
        it('Should throw error given client with null private key', () => __awaiter(this, void 0, void 0, function* () {
            const pushService = new push_1.PushService(null);
            try {
                yield pushService.create(new client_1.Client(null, 'key', 'public-key', null, null), null, null);
                throw new Error('Expected Error');
            }
            catch (error) {
                chai_1.expect(error.message).to.be.eq('Private Key cannot be null');
            }
        }));
        it('Should throw error given client with null endpoint', () => __awaiter(this, void 0, void 0, function* () {
            const pushService = new push_1.PushService(null);
            try {
                yield pushService.create(new client_1.Client(null, 'key', 'public-key', 'private-key', null), null, null);
                throw new Error('Expected Error');
            }
            catch (error) {
                chai_1.expect(error.message).to.be.eq('Endpoint cannot be null');
            }
        }));
        it('Should throw error given null channel', () => __awaiter(this, void 0, void 0, function* () {
            const pushService = new push_1.PushService(null);
            try {
                yield pushService.create(new client_1.Client(null, 'key', 'public-key', 'private-key', 'endpoint'), null, null);
                throw new Error('Expected Error');
            }
            catch (error) {
                chai_1.expect(error.message).to.be.eq('Channel cannot be null');
            }
        }));
        it('Should throw error given null payload', () => __awaiter(this, void 0, void 0, function* () {
            const pushService = new push_1.PushService(null);
            try {
                yield pushService.create(new client_1.Client(null, 'key', 'public-key', 'private-key', 'endpoint'), 'channel', null);
                throw new Error('Expected Error');
            }
            catch (error) {
                chai_1.expect(error.message).to.be.eq('Payload cannot be null');
            }
        }));
        it('Should call SubscriptionRepository#delete given webpush#sendNotification throws error with 410 status code', () => __awaiter(this, void 0, void 0, function* () {
            const subscriptionRepository = {
                delete: sinon.spy(),
                findAll: sinon.stub().returns(Promise.resolve([new subscription_1.Subscription(null, null, null)])),
            };
            const pushService = new push_1.PushService(subscriptionRepository);
            webpush.sendNotification = sinon.stub().callsFake(() => {
                const error = new Error();
                error.statusCode = 410;
                return Promise.reject(error);
            });
            yield pushService.create(new client_1.Client(null, 'key', 'public-key', 'private-key', 'endpoint'), 'channel', 'payload');
            chai_1.expect(subscriptionRepository.delete.calledOnce).to.be.true;
        }));
        it('Should throw error given webpush#sendNotification throws error', () => __awaiter(this, void 0, void 0, function* () {
            const subscriptionRepository = {
                delete: sinon.spy(),
                findAll: sinon.stub().returns(Promise.resolve([new subscription_1.Subscription(null, null, null)])),
            };
            const pushService = new push_1.PushService(subscriptionRepository);
            webpush.sendNotification = sinon.stub().returns(Promise.reject(new Error()));
            try {
                yield pushService.create(new client_1.Client(null, 'key', 'public-key', 'private-key', 'endpoint'), 'channel', 'payload');
                throw new Error('Expected Error');
            }
            catch (error) {
                chai_1.expect(error.message).to.be.not.eq('Expected Error');
            }
        }));
    });
});
//# sourceMappingURL=push.spec.js.map